
#include "mxc6xxxdio.h"

#include <linux/pci.h>


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17)
#include <linux/slab.h>
#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,2,0)
#include <linux/malloc.h>
#endif

#include <asm/io.h>
#include <asm/page.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
#include <generated/autoconf.h>   // for PCI, SMP configuration
#elif  LINUX_VERSION_CODE < KERNEL_VERSION(2,6,17)
#include <linux/config.h>   // for PCI, SMP configuration
#else
#include <linux/autoconf.h>   // for PCI, SMP configuration
#endif

#ifdef CONFIG_SMP
#ifndef __SMP__
#define __SMP__
#endif
#endif

#if defined(__SMP__)
#include <linux/smp.h>

#if defined(MODVERSIONS)
#include <linux/modversions-smp.h>
#endif

#else

#if defined(MODVERSIONS)
#include <linux/modversions.h>
#endif

#endif

#define         ADL_OP_LINTCSR                  0x4C
#define         ADL_OP_CNTRL                    0x50
#define         ADL_OP_EEPROM                   0x53

#if defined(_PCI)

#include "Plx_util.h"

#elif defined(_MPC)

extern int TotalCard;
extern int BaseIO[4];
extern int IRQ[4];

#endif

extern PCI_Info*  pci_Info[MAX_PCI_CARDS];

#if defined(_PCI)

void PLX90_ClearController(U32 dwPort)
{
	outb_p(0x00, dwPort+ADL_OP_LINTCSR);
	outb_p(0x40, dwPort+ADL_OP_EEPROM);
	outb_p(0x00, dwPort+ADL_OP_EEPROM);
}


/* Initialize the module - Register the character device */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0)

/* Initialize the module - Register the character device */
int AdlFindDevice( U16 VendorID, U16 DeviceID, U16 PciController )
{
	U32  baseAddr,lcrBase;
	U16  pci_command, subsystem_id, subvendor_id;
	U8   irq;

	static struct pci_dev *pciptr;
	int pci_info_index = 0;
	PCI_Info* temp_pci_info;

#ifdef CONFIG_PCI  // find the specified device and its configuration

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
	for( pci_info_index=0; pci_info_index < 32; ) {
	#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,10)
		if(PciController == PCI_CTRL_S593)
		{
			pciptr = pci_get_subsys(VendorID, DeviceID, PCI_ANY_ID, PCI_ANY_ID,pciptr);
		}
		else
		{	
			pciptr = pci_get_subsys(PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID,PCI_ANY_ID, pciptr);
		}
	#else
		if(PciController == PCI_CTRL_S593)
		{
			pciptr = pci_find_subsys(VendorID, DeviceID, PCI_ANY_ID, PCI_ANY_ID, pciptr);
		}
		else
		{
			pciptr = pci_find_subsys(PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, pciptr);
		}
	#endif
		if(!pciptr)
			break;
#else
		if( pcibios_present() ){
			pci_for_each_dev( pciptr ){
#endif
				pci_read_config_word( pciptr, PCI_COMMAND, &pci_command );
				//	pci_read_config_word( pciptr, PCI_SUBSYSTEM_VENDOR_ID, &subvendor_id );
				subvendor_id = pciptr->subsystem_vendor;
				//	pci_read_config_word( pciptr, PCI_SUBSYSTEM_ID, &subsystem_id );
				subsystem_id = pciptr->subsystem_device;
				switch ( PciController ) {
					case PCI_CTRL_PLX90 :
						if( ((pciptr->vendor == VendorID)&&(pciptr->device == DeviceID)) )
						{

							// 27/09/00 jeffery, allocate the memory for pci_info
							temp_pci_info = (PCI_Info*)kmalloc( sizeof(PCI_Info), GFP_KERNEL );
							if( !temp_pci_info )
								return pci_info_index; // allocate failed! just return the current index
								
							memset( temp_pci_info, '\0', sizeof( PCI_Info ));
							lcrBase = pci_resource_start( pciptr, 0 );
							baseAddr = 0;

							temp_pci_info->lcrBase = (lcrBase & ~(0x00ff));
							temp_pci_info->baseAddr = (baseAddr & ~(0x00ff));
							//temp_pci_info->Address2 = (baseAddr2 & ~(0x00ff));
							temp_pci_info->Address0 = temp_pci_info->lcrBase ;//2300
							temp_pci_info->Size0 = 0x100  ;//2300
							request_region(temp_pci_info->Address0, temp_pci_info->Size0, "mxc2300dio");
							temp_pci_info->MemSize0 = 0x100  ;//2300
							temp_pci_info->MemAddress0 = (U32)ioremap(pci_resource_start( pciptr, 1), pci_resource_len(pciptr, 1));//6300
							temp_pci_info->DRV_VERSION=DRIVER_VERSION;


							pci_enable_device(pciptr);//////////////////////

							irq = pciptr->irq;
							temp_pci_info->irqNo = irq;
							temp_pci_info->wCard = pci_info_index;
							// asigning the address into pointer array
							pci_Info[pci_info_index] = temp_pci_info;


							printk( KERN_ALERT "mxc6xxxdio-%d device found with lcrBase:[%lx]\n", \
							pci_info_index, temp_pci_info->lcrBase );


							pci_info_index ++;

						} // end of if( DeviceID == 0x7132 ) )
						break;

					default :
						break;
				} // end of switch(PciController)
    
	
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,10)
			pci_dev_put(pciptr);
#endif
		} // end of pci_for_each_dev() or for( pci_info_index=0; pci_info_index < 32; )

    		return pci_info_index; // number of the desired device
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	} // end of pcibios_present()
	else
		return -1;
#endif
#else
	return -1;
#endif
}

#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,2,0)

int AdlFindDevice( U16 VendorID, U16 DeviceID, U16 PciController )
{
	U32  modBaseAddr;
	U32  baseAddr, baseAddr2, lcrBase;

	U16  pci_command, subsystem_id, subvendor_id;
	U8   irq;

	struct pci_dev *pciptr;
	int pci_info_index = 0;
	PCI_Info* temp_pci_info;

        /*-----------------------------------------*/
#ifdef CONFIG_PCI  // find the specified device and its configuration
	if( pcibios_present() ){

		pciptr = pci_find_slot( 0, 0 );
		// find the Slot0/Func0 in Bus 0 .... to get the pci_dev List
		// assum this should exist , may find bus0/slot1 , bus0/slot2 ..... , if slot0 absent

		while( pciptr != NULL){
			pci_read_config_word( pciptr, PCI_COMMAND, &pci_command );
			pci_read_config_word( pciptr, PCI_SUBSYSTEM_VENDOR_ID, &subvendor_id );
			pci_read_config_word( pciptr, PCI_SUBSYSTEM_ID, &subsystem_id );

			switch ( PciController ) {
				/* AMCC controller Match condition */
				/*   Vendor, Device, Command  */
				/*   Command ==> bus master, memory access */
				case PCI_CTRL_PLX90 :
					if( ((pciptr->vendor == VendorID)&&(pciptr->device == DeviceID)) )
					{
						// 27/09/00 jeffery, allocate the memory for pci_info
						temp_pci_info = (PCI_Info*)kmalloc( sizeof(PCI_Info), GFP_KERNEL );
						if( !temp_pci_info ) 
							return pci_info_index; // allocate failed! just return the current index
						 	
						memset( temp_pci_info, '\0', sizeof( PCI_Info ));
						lcrBase = pciptr->base_address[1];
						baseAddr = pciptr->base_address[2];
						baseAddr2 = pciptr->base_address[3];

						temp_pci_info->lcrBase = (lcrBase & ~(0x00ff));
						temp_pci_info->baseAddr = (baseAddr & ~(0x00ff));
						//temp_pci_info->Address2 = (baseAddr2 & ~(0x00ff));
						temp_pci_info->DRV_VERSION=DRIVER_VERSION;

						irq = pciptr->irq;
						temp_pci_info->irqNo = irq;
						temp_pci_info->wCard = pci_info_index;
						// asigning the address into pointer array
						pci_Info[pci_info_index] = temp_pci_info;
						// ----------------------------------------------------------------------
						/* Set Int Polarity be Active Low, and Disable all INT at Init. */
						plx_int_control(pci_Info[pci_info_index]->lcrBase,1, 0 , 0 );
						plx_int_control(pci_Info[pci_info_index]->lcrBase,2, 0 , 0 );

						pci_info_index ++;
					} // end of if( DeviceID == 0x8134 ) )
					break;
				default :
					break;
			} // end of switch(PciController)
			pciptr = pciptr->next;
		} // end of while
		return pci_info_index; // number of the desired device
	}
	else	
		return -1;
#else
	return -1;
#endif
}
#endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0) */

#elif defined( _MPC )

/* Initialize the module - Register the character device */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0)

/* Initialize the module - Register the character device */
int AdlFindDevice(void)
{
	U8   i;
	int pci_info_index = 0;
	PCI_Info* temp_pci_info;

	/*-----------------------------------------*/
	for (i=0; i<TotalCard; i++)
	{
		if ((BaseIO[i] != 0) && (IRQ[i] != 0))
		{
			temp_pci_info = (PCI_Info*)kmalloc( sizeof(PCI_Info), GFP_KERNEL );
			if( !temp_pci_info ) 
				return pci_info_index; 
			memset( temp_pci_info, '\0', sizeof( PCI_Info ));
			temp_pci_info->baseAddr = BaseIO[i];
			temp_pci_info->irqNo = IRQ[i];
			temp_pci_info->initFlag=0x7132;
					    temp_pci_info->DRV_VERSION=DRIVER_VERSION;
			temp_pci_info->wCard = pci_info_index;
			pci_Info[pci_info_index] = temp_pci_info;
			pci_info_index ++;
		}
	}
	return pci_info_index;
}

#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,2,0)


int AdlFindDevice(void)
{
	U8   i;
	int pci_info_index = 0;
	PCI_Info* temp_pci_info;

	/*-----------------------------------------*/
	for (i=0; i<TotalCard; i++)
	{
		if ((BaseIO[i] != 0) && (IRQ[i] != 0))
		{
			temp_pci_info = (PCI_Info*)kmalloc( sizeof(PCI_Info), GFP_KERNEL );
			if( !temp_pci_info ) 
				return pci_info_index; 
			memset( temp_pci_info, '\0', sizeof( PCI_Info ));
			temp_pci_info->baseAddr = BaseIO[i];
			temp_pci_info->irqNo = IRQ[i];
			temp_pci_info->initFlag=0x7132;
			temp_pci_info->DRV_VERSION=DRIVER_VERSION;
			temp_pci_info->wCard = pci_info_index;
			pci_Info[pci_info_index] = temp_pci_info;
			pci_info_index ++;
		}
	}
	return pci_info_index;
}

#endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0) */
#endif


