//#ifndef     PLX_UTIL_C
//    #define PLX_UTIL_C
#if defined(MODVERSIONS)
#include <linux/modversions.h>
#endif

/****************************************************************************/
/*  Copyright (c) 1995-1997, ADLink Technology Inc.  All rights reserved.   */
/*									    */
/*  File Name	:   PLX_UTIL.C						    */
/*  Purpose	:   Function Library for PLX-9050 Applications		    */
/*  Date	:   5/03/1997						    */
/*  Revision	:   1.00						    */
/*  Programmer	:   P.C.Lee						    */
/*									    */
/****************************************************************************/

/****************************************************************************/
/*	Include Files							    */
/****************************************************************************/
#include <asm/io.h>
#include "Plx_util.h"

/****************************************************************************/
/* read_LCR_dword/word/byte						    */
/* write_LCR_dword/word/byte						    */
/* Functions:								    */
/* Purpose  : read or write the LCRs					    */
/* Inputs   :								    */
/*	word	n     : LC register numbers				    */
/*	word	value : write value					    */
/* Outputs  : None							    */
/* Return: (for read functions) 					    */
/*	register value							    */
/****************************************************************************/
U32	read_LCR_dword( U16 plx_lcr_ba, U16 n )
{
	return( inl_p(plx_lcr_ba+n));
}

U16	read_LCR_word( U16 plx_lcr_ba, U16 n )
{
	return( inw_p(plx_lcr_ba+n));
}

U8	read_LCR_byte( U16 plx_lcr_ba,U16 n )
{
	return( inb_p(plx_lcr_ba+n) );
}

void	write_LCR_dword( U16 plx_lcr_ba, U16 n , U32 value )
{
	outl_p(value, (plx_lcr_ba+n));
}

void	write_LCR_word( U16 plx_lcr_ba, U16 n, U16 value )
{
	outw_p(value, (plx_lcr_ba+n));
}

void	write_LCR_byte( U16 plx_lcr_ba,U16 n, U8 value )
{
	outb_p(value, (plx_lcr_ba+n) );
}


/****************************************************************************/
/*									    */
/* set_user_io_pin							    */
/* get_user_io_pin							    */
/*									    */
/* Purpose: To Set or Get User I/O Pin on the PLX 9050 chip		    */
/*									    */
/* Inputs:								    */
/*	n : I/O pin number = 0~3					    */
/*	m : control data, 3LSBs are effective (for set) 		    */
/*		bit 0 : Special / User I/O selective			    */
/*		bit 1 : Output	/ Input 				    */
/*		bit 2 :      1	/ 0  (output value if bit 1 set as output)  */
/*		value =      1	/ 0					    */
/*									    */
/* Outputs:								    */
/*     *g : for get inport data (for get)				    */
/*									    */
/****************************************************************************/
static U16 _uiomask1[4] = {0xFFF8, 0xFFC7, 0xFE3F, 0xF1FF };
static U16 _uiomask2[4] = {0x0007, 0x0038, 0x01C0, 0x0E00 };

U16 set_user_io_pin( U16 plx_lcr_ba,U16 n , U16 m )
{
	U16 value;
	m &= 0x07;		/* mask higher bits */
	m = m << (3*n);
	value = inw_p((U16)( plx_lcr_ba + PLX_CNTRL) );
	value = (value & _uiomask1[n]) | m;
	outw_p( value, (U16)( plx_lcr_ba + PLX_CNTRL) );
	return	PLX_FUNCTION_OK;
}

/*--------------------------------------------------------------------------*/
U16 get_user_io_pin( U16 plx_lcr_ba, U16 n, U16 *g )
{
	U16 value;
	value = inw_p((plx_lcr_ba + PLX_CNTRL) );
	value = (value & _uiomask2[n]) >> (3*n) ;
	value = value >> 2;
	*g = value;
	return	PLX_FUNCTION_OK;
}


/****************************************************************************/
/*	Functions about read / write EEPROM of PLX-9050 		    */
/****************************************************************************/
/*									    */
/* read_eeprom								    */
/* write_eeprom 							    */
/* write_eeprom_enanle							    */
/* write_eeprom_disable 						    */
/*									    */
/* Purpose: To read or write EEPROM value of the PLX 9050 chip		    */
/*									    */
/* Inputs:								    */
/*	word n	: EEPROM register number				     */
/*	byte ro: the data write to EEPROM				    */
/*									    */
/* Outputs:								    */
/*	byte *ri: the EEPROM data read out value			    */
/*									    */
/* Retrun:								    */
/*	ERR_PLX_EEPROM_FAILURE : cannot access EEPROM or failure	    */
/*	SUCCESSFUL  :  read or write EEPROM OK				    */
/*									    */
/* Used locol functions:						    */
/*	_init_cmd(), _clk1(), _clk0(), _delay1(), _send_ewds(), 	    */
/*	_read_cmd(),							    */
/*									    */
/****************************************************************************/
// EEPROM of CNTRL register of PLC9050 chip
//  bits#
//    24  SK  0  1  0  1  0
//    25  CS  1  1  1  1  0
//    26  DI  0  0  1  1  0
//    27  DO  X  X  X  X  X
//	     02 03 06 07 00

#define DISABLE 0x00
#define ENABLE	0x02
#define DIL_SK0 0x02
#define DIL_SK1 0x03
#define DIH_SK0 0x06
#define DIH_SK1 0x07
/*
static void _delay1( U16 len )
{
	register int k;
	for ( k = 0; k < len; k++);
}

static void _init_cmd(U16 ba)
{
	write_LCR_byte(ba,PLX_EEPROM, 0x07  );
	write_LCR_byte(ba,PLX_EEPROM, 0x02  );
	write_LCR_byte(ba,PLX_EEPROM, 0x00  );
}
*/
static void _clk1(U16 ba)	 /* DI==1, Sk==0->1  */
{
	write_LCR_byte(ba,PLX_EEPROM, DIH_SK0 );
	write_LCR_byte(ba,PLX_EEPROM, DIH_SK1 );
}

static void _clk0(U16 ba)	 /* DI==0, Sk==0->1  */
{
	write_LCR_byte(ba,PLX_EEPROM, DIL_SK0 );
	write_LCR_byte(ba,PLX_EEPROM, DIL_SK1 );
}



U16 check_eeprom_valid( U16 plx_lcr_ba )
{
	U8 b;
	b = (read_LCR_byte( plx_lcr_ba , PLX_EEPROM ) >> 4 ) & 0x01;
	if( b==1 )	
		return ( PLX_EEPROM_VALID );
	else		
		return ( PLX_EEPROM_NOT_VALID );
}

/*--------------------------------------------------------------------------*/
U16 read_eeprom( U16 ba, U16 adr , U16 *data )
{
	U8	b;
	U16	tmp,i;
	adr = adr /2;
	if(check_eeprom_valid(ba) != PLX_EEPROM_VALID )
			return PLX_EEPROM_NOT_VALID;
	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	write_LCR_byte(ba,PLX_EEPROM, ENABLE  );
	_clk1(ba);		/* Send Start bit	*/
	_clk1(ba);_clk0(ba);	/* Send Read Op Code	*/
				/* Send 6 bits address	*/
	for ( i = 0; i < 6 ; i++)
	{
	    if ( ((adr << i) & 0x20) != 0)
		_clk1(ba);
	    else
		_clk0(ba);
	}

	tmp=0;
	for ( i = 0; i < 16 ; i++)
	{
		_clk0(ba);
		b = read_LCR_byte(ba,PLX_EEPROM );
		b = (b>>3) & 0x01;
		tmp = tmp | ( b << (15-i) );
	}
	*data = tmp;

	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	return	PLX_FUNCTION_OK;
}
/*--------------------------------------------------------------------------*/
U16 write_eeprom( U16 ba, U16 adr , U16 data )
{
	I16	 i;
	U8	b;

	adr = adr /2;
	if( check_eeprom_valid(ba) != PLX_EEPROM_VALID )
			return PLX_EEPROM_NOT_VALID;

	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	write_LCR_byte(ba,PLX_EEPROM, ENABLE  );

	_clk1(ba);		/* Send Start bit	*/
	_clk0(ba);_clk1(ba);	/* Send Write Op Code	*/
				/* Send 6 bits address	*/
	for ( i = 0; i < 6 ; i++)
	{
	    if ( ((adr << i) & 0x20) != 0)
		_clk1(ba);
	    else
		_clk0(ba);
	}
				/* Write 16 bits data	*/
	for ( i = 15; i >=0  ; i--)
	{
		if( (data>>i) & 0x01 )
			_clk1(ba);
		else
			_clk0(ba);
	}

	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	write_LCR_byte(ba,PLX_EEPROM, ENABLE  );

	do			/* Wait for Write ready (need 15 ms)   */
	{
		b = read_LCR_byte(ba,PLX_EEPROM );
		b = (b>>3) & 0x01;
	} while( b==0 );
	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	return	PLX_FUNCTION_OK;
}
/*--------------------------------------------------------------------------*/
U16 write_eeprom_enable( U16 ba )
{
	U16 i;

	if( check_eeprom_valid(ba) != PLX_EEPROM_VALID )
			return PLX_EEPROM_NOT_VALID;

	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	write_LCR_byte(ba,PLX_EEPROM, ENABLE  );

	_clk1(ba);		/* Send Start bit	*/
	_clk0(ba);_clk0(ba);	/* Send WEN  Op Code	*/
				/* Send 6 bits address	*/
	for ( i = 0; i < 6 ; i++)	_clk1(ba);

	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	return	PLX_FUNCTION_OK;
}
/*--------------------------------------------------------------------------*/
U16 write_eeprom_disable( U16 ba )
{
	I16 i;

	if( check_eeprom_valid(ba) != PLX_EEPROM_VALID )
			return PLX_EEPROM_NOT_VALID;

	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	write_LCR_byte(ba,PLX_EEPROM, ENABLE  );

	_clk1(ba);		/* Send Start bit	*/
	_clk0(ba);_clk0(ba);	/* Send WDS  Op Code	*/
				/* Send 6 bits address	*/
	for ( i = 0; i < 6 ; i++)	_clk0(ba);

	write_LCR_byte(ba,PLX_EEPROM, DISABLE );
	return	PLX_FUNCTION_OK;
}

/****************************************************************************/
/* Function: reload_eeprom						    */
/* Purpose: Reload the PCI configuration registers from EEPROM		    */
/* I/O	  : None							    */
/****************************************************************************/
void reload_eeprom( U16 lcr_ba )
{
	write_LCR_byte(lcr_ba,PLX_EEPROM, 0x20 );   /* set bit #29 of CNTRL */
	write_LCR_byte(lcr_ba,PLX_EEPROM, 0x00 );   /* clr bit #29 of CNTRL */
}

/****************************************************************************/
/* Function: software_reset						    */
/* Purpose: Reset PCI 9050 and issue to the local bus			    */
/* I/O	  : None							    */
/****************************************************************************/
void software_reset( U16 lcr_ba )
{
	write_LCR_byte(lcr_ba,PLX_EEPROM, 0x40 );  /* set bit #30 of CNTRL */
	write_LCR_byte(lcr_ba,PLX_EEPROM, 0x00 );
}

/****************************************************************************/
/* Function: software_reset						    */
/* Purpose: Reset PCI 9050 and issue to the local bus			    */
/* Return Value :							    */
/*	EEPROM_VALID							    */
/****************************************************************************/

/****************************************************************************/
/* Function: plx_int_control						    */
/* Purpose: Control PLX 9050 interrupt					    */
/* Input :								    */
/*	word	 n : Local interrupt channel number (n=1 or 2)		     */
/*		    n=0 or others to control the PCI interrupt status	    */
/*	word	 enable : 0 for disable int., 1 for enable int. 	     */
/*	word	act_h : active level					    */
/****************************************************************************/
void plx_int_control( U16 plx_lcr_ba, U16 n, U16 enable,  U16 act_h )
{
	U8 b;

	b = read_LCR_byte( plx_lcr_ba, PLX_INTCSR ) ;
	switch( n )
	{
		case 0:
		default:
			if( enable )
				b = b | 0x40;
			else
				b = b & 0xBF;
			break;
		case 1:
			if( enable )
				b |= 0x01;
			else
				b &= 0xFE;
			if( act_h  )	
				b |= 0x02;
			else
				b &= 0xFD;
			break;
		case 2:
			if( enable )
				b |= 0x08;
			else
				b &= 0xF7;
			if( act_h  )
				b |= 0x10;
			else
				b &= 0xEF;
			break;
	}

	write_LCR_byte( plx_lcr_ba , PLX_INTCSR , b );
}

/****************************************************************************/
/* Function: interrupt_status						    */
/* Purpose: Check PLX 9050 interrupt status				    */
/* Input :								    */
/*	word	 n : Local interrupt channel number (n=1 or 2)		     */
/* Return Value :							    */
/*	PLX_INT_ACTIVE							    */
/*	PLX_INT_NOT_ACTIVE						    */
/****************************************************************************/
U16 interrupt_status( U16 plx_lcr_ba,U16 n )
{
	U8	b;
	b = read_LCR_byte( plx_lcr_ba, PLX_INTCSR ) ;
	if( n==2 )	
		b = b >> 3;
	b = (b>>2) & 0x01;
	return b;
}

U16 interrupt_status2( U16 plx_lcr_ba )
{
	U8	b;
	b = read_LCR_byte( plx_lcr_ba, PLX_INTCSR ) ;
	return b;
}
/****************************************************************************/
/* Function: software_interrupt 					    */
/* Purpose: Generate software interrupt 				    */
/* I/O	  : None							    */
/***************************************************************************/
void generate_sw_int( U16 plx_lcr_ba )
{
	U8 b;
	b = inb_p( plx_lcr_ba+ PLX_INTCSR );
	outb_p( (b | 0x80), (U16)(plx_lcr_ba+ PLX_INTCSR) );
}

/****************************************************************************/
void clear_sw_int( U16 plx_lcr_ba )
{
	U8 b;

	b = inb_p ((U16)( plx_lcr_ba+ PLX_INTCSR) );
	outb_p( (b & 0x7F), plx_lcr_ba+ PLX_INTCSR );
}

