#ifndef MODULE
#define MODULE
#endif

#include <linux/version.h>

//#include <linux/init.h>

#include <linux/module.h>   /* Specifically, a module */
#include <linux/kernel.h>   /* We're doing kernel work */

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
#include <generated/autoconf.h>   // for PCI, SMP configuration
#elif  LINUX_VERSION_CODE < KERNEL_VERSION(2,6,17)
#include <linux/config.h>   // for PCI, SMP configuration
#else
#include <linux/autoconf.h>   // for PCI, SMP configuration
#endif

//#include <linux/malloc.h>
//#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/pci.h>
#include <linux/fs.h>
//#include <linux/ioport.h>
#include <asm/io.h>
//#include <asm/page.h>  // for PAGE_MASK, PAGE_SHIFT
#include <linux/delay.h>
#include <asm/uaccess.h>
#include <linux/interrupt.h>

#include <asm/page.h>
#include <asm/pgtable.h>

#if LINUX_VERSION_CODE > KERNEL_VERSION(4,11,12)
#include <linux/uaccess.h>
#endif

#ifdef CONFIG_SMP
#ifndef __SMP__
#define __SMP__
#endif
#endif

#if defined(__SMP__)

#ifndef CONFIG_X86_LOCAL_APIC
#define CONFIG_X86_LOCAL_APIC
#endif
#include <linux/smp.h>

#if defined(MODVERSIONS)
#include <linux/modversions-smp.h>
#endif

#else

#if defined(MODVERSIONS)
#include <linux/modversions.h>
#endif

#endif

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17)
#include <linux/slab.h>
#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,2,0)
#include <linux/malloc.h>
#endif


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,2,0)
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
#include <unistd.h>
#endif
#endif

#include "commdef_mxc6xxxdio.h"
#include "mxc6xxxdio.h"
#include "Plx_util.h"

#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,20) )
#define PTE_OFFSET pte_offset
#else
#define PTE_OFFSET pte_offset_kernel
#endif




#if defined(_MPC)
static int TotalCard=0;
static int BaseIO[4] = {0,0,0,0};
static int IRQ[4] = {0,0,0,0};

MODULE_PARM(TotalCard, "1-1i");
MODULE_PARM(BaseIO, "1-4i");
MODULE_PARM(IRQ, "1-4i");
#endif
			
/* The name for our device, as it will appear in
 * /proc/devices */

MODULE_LICENSE("GPL");


/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>*/
#if defined(_PCI)
#define 	DEVICE_NAME 	"pmxc6xxxdio"
#define 	ADL_VENDOR_ID 0x144A
#define 	ADL_DEVICE_ID 	0x7132
#elif defined (_MPC)
#define 	DEVICE_NAME 	"mpc8164"
#endif
/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>*/


#define	ADL_PT0_DO				0x00
#define	ADL_PT0_DI				0x00
#define	ADL_PT0_COS_CS		0x08
#define	ADL_PT0_COS_EN_LATCH	0x0C
#define	ADL_PT0_DO_RD			0x100   // DO read back
#define	ADL_PT0_COS_EN_RD	0x104
PCI_Info *pci_Info[MAX_PCI_CARDS]; // instance for every device_open

static int device_number = 0; // return by AdlFindDevice(...) in init_module
static int MAJOR = 0;

int pmxc6xxxdio_open(struct inode *inode, struct file *file)
{
	int dev_index;
	PCI_Info* ppci_info;

	dev_index = MINOR(inode->i_rdev);
	ppci_info = pci_Info[dev_index];

	if( dev_index >= device_number )
		return -1;


	if( ppci_info->Reference > 0 ){ // disable multi-open now.
		ppci_info->Reference ++; //multi-open supported
		file->private_data = (void*)ppci_info;
		return 0;
	}

	// Alloc will be copied from Libraries
	ppci_info->initFlag = 0x7132;
	file->private_data = (void*)ppci_info;

	ppci_info->Reference ++;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
	MOD_INC_USE_COUNT;
#endif
	return 0;
}

int pmxc6xxxdio_release(struct inode *inode,struct file *file)
{
	PCI_Info* ppci_info = (PCI_Info*)(file->private_data);

	ppci_info->Reference --;
	ppci_info->initFlag = FALSE;


#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
	MOD_DEC_USE_COUNT;
#endif
	return 0;
}
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,35)
long pmxc6xxxdio_ioctl( struct file *file,
    unsigned int ioctl_num,/* The number of the ioctl */
    unsigned long ioctl_param) /* The parameter to it */
#else 
int pmxc6xxxdio_ioctl( struct inode *inode,  struct file *file,
	unsigned int ioctl_num,/* The number of the ioctl */
	unsigned long ioctl_param) /* The parameter to it */
#endif
{

	PCI_Info* ppci_info = (PCI_Info*)(file->private_data);
	void* temp_param;
	U32 errcode;
	U16 wValue;

	//---------------------------------------
	switch ( ioctl_num ){

		case DAS_IOC_GET_GPI :	
			if( ppci_info->lcrBase != 0 )
			{	
				temp_param = (void *)kmalloc( sizeof( unsigned long ), GFP_ATOMIC );
				errcode=copy_from_user ((void *)temp_param, (const void*)ioctl_param, sizeof( unsigned long ));
				wValue =  readw((U32*) (ppci_info->MemAddress0 + ADL_PT0_DI));
				*((unsigned long *)temp_param) = (unsigned long)wValue;
				errcode=copy_to_user((void *)ioctl_param, (const void *)temp_param, sizeof( unsigned long ));
				kfree( temp_param );
			}
			return 0;
		case DAS_IOC_SET_GPO :	
			if( ppci_info->lcrBase != 0 )
			{
				temp_param = (void *)kmalloc( sizeof( unsigned long ), GFP_ATOMIC );
				errcode=copy_from_user ((void *)temp_param, (const void*)ioctl_param, sizeof( unsigned long  ));
				wValue = (*((unsigned long *)temp_param) & 0xFFFF);
				writew(wValue , (U32*)(ppci_info->MemAddress0 + ADL_PT0_DO));
				wValue = readw( (U32*)(ppci_info->MemAddress0 + ADL_PT0_DO_RD));
				*((unsigned long *)temp_param) = (unsigned long )wValue;
				errcode=copy_to_user((void *)ioctl_param, (const void *)temp_param, sizeof( unsigned long  ));
				kfree( temp_param );
			}
			return 0;
		case DAS_IOC_GET_GPO :	
			if( ppci_info->lcrBase != 0 )
			{
				temp_param = (void *)kmalloc( sizeof( unsigned long  ), GFP_ATOMIC );
				errcode=copy_from_user ((void *)temp_param, (const void*)ioctl_param, sizeof( unsigned long  ));
				wValue =  readw( (U32*)(ppci_info->MemAddress0 + ADL_PT0_DO_RD));
				*((unsigned long *)temp_param) = (unsigned long )wValue;
				errcode=copy_to_user((void *)ioctl_param, (const void *)temp_param, sizeof( unsigned long  ));
				kfree( temp_param );
			}
			return 0;		
		default :
			return -1;
	}
	/*---------------------------------------*/
	return 0;
}

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,35)
struct file_operations fops = {
	unlocked_ioctl: pmxc6xxxdio_ioctl,   /*ioctl */
	open: pmxc6xxxdio_open,
	release: pmxc6xxxdio_release  /* a.k.a. close */
};
#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0)
struct file_operations fops = {
	ioctl: pmxc6xxxdio_ioctl,   /*ioctl */
	open: pmxc6xxxdio_open,
	release: pmxc6xxxdio_release  /* a.k.a. close */
};

#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,2,0)
struct file_operations fops = {
	NULL,   /* seek */
	NULL, // pmxc6xxxdio_read,
	NULL, // pmxc6xxxdio_write,
	NULL,   /* readdir */
	NULL,   /* select */
	pmxc6xxxdio_ioctl,   /*ioctl */
	NULL,   /* mmap */
	pmxc6xxxdio_open,
	NULL,  /* flush */
	pmxc6xxxdio_release  /* a.k.a. close */
};

#endif

int pmxc6xxxdio_init( void )
{
	int rel_val, i;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
       EXPORT_NO_SYMBOLS; // don't export any symbol
#endif
	/* substitute for AdlFindDeviceNext_loop */
	for( i=0; i<MAX_PCI_CARDS; i++)
		pci_Info[i]=NULL;
#if defined(_PCI)
	device_number = AdlFindDevice( ADL_VENDOR_ID, ADL_DEVICE_ID, PCI_CTRL_PLX90 );
#elif defined(_MPC)
	device_number = AdlFindDevice();
#endif

	if( device_number <= 0 ){// find no desired card
		return -1; // DAS_ERR_NO_DEVICE; // this will cause the error in insmod
	}

	rel_val = register_chrdev(MAJOR,DEVICE_NAME,&fops);

	if (rel_val<0){
		return rel_val;
	}
	// additional initialization

	MAJOR = rel_val;
	printk ( KERN_ALERT "Drv[mxc6xxxdio]: Registering the character device: %s successful \n", DEVICE_NAME );

	return PCIBIOS_SUCCESSFUL;

}


/* Cleanup - unregister the appropriate file from /proc */
void pmxc6xxxdio_exit(void){

	int i;

	for( i = 0; i < device_number; i++ ){

	// free_irq
	if( pci_Info[i]->irqNo != 0 && pci_Info[i]->IsrEnable == TRUE) // pci bios asign some IRQ for 8164 , free the IRQ
		free_irq( pci_Info[i]->irqNo , pci_Info[i]);
	iounmap((U32*)pci_Info[i]->MemAddress0);
	// 27/09/00 jeffrey , free the memory allocated in AdlFindDevice(...)
	if( pci_Info[i] != NULL )
		kfree( pci_Info[i] );
	}

	device_number = 0;

	//ret = unregister_chrdev(MAJOR, DEVICE_NAME);
	unregister_chrdev(MAJOR, DEVICE_NAME);
}


module_init( pmxc6xxxdio_init );
module_exit( pmxc6xxxdio_exit );
